]> git.rkrishnan.org Git - pihpsdr.git/commitdiff
Some editing to converge different pihpsdr versions
authorc vw <dl1ycf@darc.de>
Fri, 28 May 2021 12:02:25 +0000 (14:02 +0200)
committerc vw <dl1ycf@darc.de>
Fri, 28 May 2021 12:02:25 +0000 (14:02 +0200)
23 files changed:
Makefile
audio.c
audio.h
configure.c
cw_menu.c
exit_menu.c
ext.c
hpsdrsim.c
hpsdrsim.h
iambic.h
midi2.c
midi3.c
midi_menu.c
new_protocol.c
old_protocol.c
portaudio.c
pulseaudio.c
receiver.c
receiver.h
rigctl.c
rx_panadapter.c
step_menu.c
vfo_menu.c

index 5c5847ac4bc10540a205d8784c42a1d7030942a8..155fad4c6e86eaf0b3c1b09750e548d974c870cd 100644 (file)
--- a/Makefile
+++ b/Makefile
@@ -26,10 +26,10 @@ MIDI_INCLUDE=MIDI
 # USBOZY_INCLUDE=USBOZY
 
 # uncomment the line to below include support local CW keyer
-LOCALCW_INCLUDE=LOCALCW
+#LOCALCW_INCLUDE=LOCALCW
 
 # uncomment the line below for SoapySDR
-SOAPYSDR_INCLUDE=SOAPYSDR
+#SOAPYSDR_INCLUDE=SOAPYSDR
 
 # uncomment the line to below include support for sx1509 i2c expander
 #SX1509_INCLUDE=sx1509
@@ -38,8 +38,10 @@ SOAPYSDR_INCLUDE=SOAPYSDR
 #STEMLAB_DISCOVERY=STEMLAB_DISCOVERY
 
 # uncomment the line below to include support for STEMlab discovery (WITHOUT AVAHI)
-STEMLAB_DISCOVERY=STEMLAB_DISCOVERY_NOAVAHI
+#STEMLAB_DISCOVERY=STEMLAB_DISCOVERY_NOAVAHI
 
+# uncomment to get ALSA audio module on Linux (default is now to use pulseaudio)
+#AUDIO_MODULE=ALSA
 
 # uncomment the line below for various debug facilities
 #DEBUG_OPTION=-D DEBUG
@@ -49,9 +51,6 @@ STEMLAB_DISCOVERY=STEMLAB_DISCOVERY_NOAVAHI
 # very early code not included yet
 #SERVER_INCLUDE=SERVER
 
-# uncomment to get ALSA audio module on Linux
-#AUDIO_MODULE=ALSA
-
 CFLAGS?= -O -Wno-deprecated-declarations
 PKG_CONFIG = pkg-config
 
@@ -230,12 +229,12 @@ ifeq ($(UNAME_S), Linux)
 SYSLIBS=-lrt
 endif
 ifeq ($(UNAME_S), Darwin)
-SYSLIBS=-framework IOkit
+SYSLIBS=-framework IOKit
 endif
 
 OPTIONS=$(SMALL_SCREEN_OPTIONS) $(MIDI_OPTIONS) $(PURESIGNAL_OPTIONS) $(REMOTE_OPTIONS) $(USBOZY_OPTIONS) \
        $(GPIO_OPTIONS) $(GPIOD_OPTIONS)  $(SOAPYSDR_OPTIONS) $(LOCALCW_OPTIONS) \
-       $(STEMLAB_OPTIONS) $(PTT_OPTIONES) $(SERVER_OPTIONS) $(AUDIO_OPTIONS) \
+       $(STEMLAB_OPTIONS) $(PTT_OPTIONES) $(SERVER_OPTIONS) $(AUDIO_OPTIONS) $(GPIO_OPTIONS) \
        -D GIT_DATE='"$(GIT_DATE)"' -D GIT_VERSION='"$(GIT_VERSION)"' $(DEBUG_OPTION)
 
 LIBS=  -lm -lwdsp -lpthread $(SYSLIBS) $(AUDIO_LIBS) $(USBOZY_LIBS) $(GTKLIBS) \
diff --git a/audio.c b/audio.c
index ff200fabe6c587d352882dc07309e018ca15879e..7c54ed8e7b63c630052a06b80926627184a1869a 100644 (file)
--- a/audio.c
+++ b/audio.c
@@ -50,6 +50,7 @@
 
 int audio = 0;
 int mic_buffer_size = 720; // samples (both left and right)
+static GMutex audio_mutex;
 
 static snd_pcm_t *record_handle=NULL;
 static snd_pcm_format_t record_audio_format;
@@ -193,10 +194,13 @@ g_print("audio_open_input: %s\n",transmitter->microphone_name);
   g_print("audio_open_input: hw=%s\n",hw);
 
   for(i=0;i<FORMATS;i++) {
+    g_mutex_lock(&audio_mutex);
     if ((err = snd_pcm_open (&record_handle, hw, SND_PCM_STREAM_CAPTURE, SND_PCM_ASYNC)) < 0) {
       g_print("audio_open_input: cannot open audio device %s (%s)\n",
               hw,
               snd_strerror (err));
+      record_handle=NULL;
+      g_mutex_unlock(&audio_mutex);
       return err;
     }
 g_print("audio_open_input: handle=%p\n",record_handle);
@@ -204,6 +208,7 @@ g_print("audio_open_input: handle=%p\n",record_handle);
 g_print("audio_open_input: trying format %s (%s)\n",snd_pcm_format_name(formats[i]),snd_pcm_format_description(formats[i]));
     if ((err = snd_pcm_set_params (record_handle,formats[i],SND_PCM_ACCESS_RW_INTERLEAVED,channels,rate,soft_resample,latency)) < 0) {
       g_print("audio_open_input: snd_pcm_set_params failed: %s\n",snd_strerror(err));
+      g_mutex_unlock(&audio_mutex);
       audio_close_input();
       continue;
     } else {
@@ -215,6 +220,7 @@ g_print("audio_open_input: using format %s (%s)\n",snd_pcm_format_name(formats[i
 
   if(i>=FORMATS) {
     g_print("audio_open_input: cannot find usable format\n");
+    g_mutex_unlock(&audio_mutex);
     audio_close_input();
     return err;
   }
@@ -240,6 +246,7 @@ g_print("audio_open_input: allocating ring buffer\n");
   mic_ring_buffer=(float *) g_new(float,MICRINGLEN);
   mic_ring_read_pt = mic_ring_write_pt=0;
   if (mic_ring_buffer == NULL) {
+    g_mutex_unlock(&audio_mutex);
     audio_close_input();
     return -1;
   }
@@ -249,10 +256,12 @@ g_print("audio_open_input: creating mic_read_thread\n");
   mic_read_thread_id = g_thread_try_new("microphone",mic_read_thread,NULL,&error);
   if(!mic_read_thread_id ) {
     g_print("g_thread_new failed on mic_read_thread: %s\n",error->message);
+    g_mutex_unlock(&audio_mutex);
     audio_close_input();
     return -1;
   }
 
+  g_mutex_unlock(&audio_mutex);
   return 0;
 }
 
@@ -271,14 +280,15 @@ g_print("audio_close_output: rx=%d handle=%p buffer=%p\n",rx->id,rx->playback_ha
 }
 
 void audio_close_input() {
-  void *p;
 g_print("audio_close_input\n");
   running=FALSE;
+  g_mutex_lock(&audio_mutex);
   if(mic_read_thread_id!=NULL) {
 g_print("audio_close_input: wait for thread to complete\n");
     g_thread_join(mic_read_thread_id);
     mic_read_thread_id=NULL;
   }
+  g_mutex_lock(&audio_mutex);
   if(record_handle!=NULL) {
 g_print("audio_close_input: snd_pcm_close\n");
     snd_pcm_close (record_handle);
@@ -289,19 +299,10 @@ g_print("audio_close_input: free mic buffer\n");
     g_free(mic_buffer);
     mic_buffer=NULL;
   }
-  //
-  // We do not want to do a mutex lock/unlock for every single mic sample
-  // accessed. Since only the ring buffer is maintained by the functions
-  // audio_get_next_mic_sample() and in the "mic read thread",
-  // it is more than enough to wait 2 msec after setting mic_ring_buffer to NULL
-  // before actually releasing the storage.
-  //
   if (mic_ring_buffer != NULL) {
-    p=mic_ring_buffer;
-    mic_ring_buffer=NULL;
-    usleep(2);
-    g_free(p);
+    g_free(mic_ring_buffer);
   }
+  g_mutex_unlock(&audio_mutex);
 }
 
 //
@@ -641,18 +642,18 @@ g_print("mic_read_thread: exiting\n");
 float audio_get_next_mic_sample() {
   int newpt;
   float sample;
+  g_mutex_lock(&audio_mutex);
   if ((mic_ring_buffer == NULL) || (mic_ring_read_pt == mic_ring_write_pt)) {
     // no buffer, or nothing in buffer: insert silence
     sample=0.0;
   } else {
-    // the "existence" of the ring buffer is now guaranteed for 1 msec,
-    // see audio_close_input(),
     newpt = mic_ring_read_pt+1;
     if (newpt == MICRINGLEN) newpt=0;
     sample=mic_ring_buffer[mic_ring_read_pt];
     // atomic update of read pointer
     mic_ring_read_pt=newpt;
   }
+  g_mutex_unlock(&audio_mutex);
   return sample;
 }
 
@@ -667,6 +668,7 @@ void audio_get_cards() {
 
 g_print("audio_get_cards\n");
 
+  g_mutex_init(&audio_mutex);
   n_input_devices=0;
   n_output_devices=0;
 
diff --git a/audio.h b/audio.h
index 398b8c241a3d1a092e047ed3627e882e1971cf1f..5f6832df105351ef2dcaac6f7cd230d9e6cc2533 100644 (file)
--- a/audio.h
+++ b/audio.h
@@ -36,15 +36,13 @@ extern int n_input_devices;
 extern AUDIO_DEVICE input_devices[MAX_AUDIO_DEVICES];
 extern int n_output_devices;
 extern AUDIO_DEVICE output_devices[MAX_AUDIO_DEVICES];
-extern GMutex audio_mutex;
-extern gint local_microphone_buffer_size;
 
 extern int audio_open_input();
 extern void audio_close_input();
 extern int audio_open_output(RECEIVER *rx);
 extern void audio_close_output(RECEIVER *rx);
 extern int audio_write(RECEIVER *rx,float left_sample,float right_sample);
-extern int cw_audio_write(RECEIVER *rx,float sample);
+extern int cw_audio_write(RECEIVER *rx, float sample);
 extern void audio_get_cards();
 char * audio_get_error_string(int err);
 float  audio_get_next_mic_sample();
index e583b970d14923bf2cc8ebd4efb378ed09e8dc59..a949414480143b34ba79169980bd7571ed0b1b8c 100644 (file)
@@ -88,60 +88,49 @@ void configure_gpio(GtkWidget *parent) {
 
   if (max_encoders > 0) {
 
-  grid=gtk_grid_new();
-  gtk_grid_set_column_homogeneous(GTK_GRID(grid),FALSE);
-  gtk_grid_set_row_homogeneous(GTK_GRID(grid),TRUE);
-  gtk_grid_set_column_spacing (GTK_GRID(grid),2);
-  gtk_grid_set_row_spacing (GTK_GRID(grid),2);
+    grid=gtk_grid_new();
+    gtk_grid_set_column_homogeneous(GTK_GRID(grid),FALSE);
+    gtk_grid_set_row_homogeneous(GTK_GRID(grid),TRUE);
+    gtk_grid_set_column_spacing (GTK_GRID(grid),2);
+    gtk_grid_set_row_spacing (GTK_GRID(grid),2);
 
 
 /*
-  widget=gtk_label_new(NULL);
-  gtk_label_set_markup (GTK_LABEL(widget), "<span foreground=\"#ff0000\"><b>Note: Pin number now use Broadcom GPIO</b></span>");
-  gtk_grid_attach(GTK_GRID(grid),widget,col,row,6,1);
+    widget=gtk_label_new(NULL);
+    gtk_label_set_markup (GTK_LABEL(widget), "<span foreground=\"#ff0000\"><b>Note: Pin number now use Broadcom GPIO</b></span>");
+    gtk_grid_attach(GTK_GRID(grid),widget,col,row,6,1);
 
-  row++;
-  col=0;
+    row++;
+    col=0;
 */
-  widget=gtk_label_new("");
-  gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
-  col++;
-
-  widget=gtk_label_new(NULL);
-  gtk_label_set_markup (GTK_LABEL(widget), controller==CONTROLLER2_V2?"<b>Bottom Encoder</b>":"<b>Encoder</b>");
-  gtk_grid_attach(GTK_GRID(grid),widget,col,row,2,1);
-  col+=2;
+    widget=gtk_label_new("");
+    gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
+    col++;
 
-  if(controller==CONTROLLER2_V2) {
     widget=gtk_label_new(NULL);
-    gtk_label_set_markup (GTK_LABEL(widget), "<b>Top Encoder</b>");
+    gtk_label_set_markup (GTK_LABEL(widget), controller==CONTROLLER2_V2?"<b>Bottom Encoder</b>":"<b>Encoder</b>");
     gtk_grid_attach(GTK_GRID(grid),widget,col,row,2,1);
     col+=2;
-  }
-
-  widget=gtk_label_new(NULL);
-  gtk_label_set_markup (GTK_LABEL(widget), "<b>Switch</b>");
-  gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
 
-  row++;
-  col=0;
+    if(controller==CONTROLLER2_V2) {
+      widget=gtk_label_new(NULL);
+      gtk_label_set_markup (GTK_LABEL(widget), "<b>Top Encoder</b>");
+      gtk_grid_attach(GTK_GRID(grid),widget,col,row,2,1);
+      col+=2;
+    }
 
-  widget=gtk_label_new(NULL);
-  gtk_label_set_markup (GTK_LABEL(widget), "<b>ID</b>");
-  gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
-  col++;
+    widget=gtk_label_new(NULL);
+    gtk_label_set_markup (GTK_LABEL(widget), "<b>Switch</b>");
+    gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
 
-  widget=gtk_label_new(NULL);
-  gtk_label_set_markup (GTK_LABEL(widget), "<b>Gpio A</b>");
-  gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
-  col++;
+    row++;
+    col=0;
 
-  widget=gtk_label_new(NULL);
-  gtk_label_set_markup (GTK_LABEL(widget), "<b>Gpio B</b>");
-  gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
-  col++;
+    widget=gtk_label_new(NULL);
+    gtk_label_set_markup (GTK_LABEL(widget), "<b>ID</b>");
+    gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
+    col++;
 
-  if(controller==CONTROLLER2_V2) {
     widget=gtk_label_new(NULL);
     gtk_label_set_markup (GTK_LABEL(widget), "<b>Gpio A</b>");
     gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
@@ -151,57 +140,68 @@ void configure_gpio(GtkWidget *parent) {
     gtk_label_set_markup (GTK_LABEL(widget), "<b>Gpio B</b>");
     gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
     col++;
-  }
 
-  widget=gtk_label_new(NULL);
-  gtk_label_set_markup (GTK_LABEL(widget), "<b>Gpio</b>");
-  gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
-  col++;
+    if(controller==CONTROLLER2_V2) {
+      widget=gtk_label_new(NULL);
+      gtk_label_set_markup (GTK_LABEL(widget), "<b>Gpio A</b>");
+      gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
+      col++;
 
-  row++;
-  col=0;
+      widget=gtk_label_new(NULL);
+      gtk_label_set_markup (GTK_LABEL(widget), "<b>Gpio B</b>");
+      gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
+      col++;
+    }
 
-  for(i=0;i<max_encoders;i++) {
     widget=gtk_label_new(NULL);
-    gchar id[16];
-    g_sprintf(id,"<b>%d</b>",i);
-    gtk_label_set_markup (GTK_LABEL(widget), id);
+    gtk_label_set_markup (GTK_LABEL(widget), "<b>Gpio</b>");
     gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
     col++;
 
-    widget=gtk_spin_button_new_with_range (0.0,28.0,1.0);
-    gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget),encoders[i].bottom_encoder_address_a);
-    gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
-    col++;
-    
-    widget=gtk_spin_button_new_with_range (0.0,28.0,1.0);
-    gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget),encoders[i].bottom_encoder_address_b);
-    gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
-    col++;
-    
-    if(controller==CONTROLLER2_V2 && i<(max_encoders-1)) {
-      widget=gtk_spin_button_new_with_range (0.0,28.0,1.0);
-      gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget),encoders[i].top_encoder_address_a);
+    row++;
+    col=0;
+
+    for(i=0;i<max_encoders;i++) {
+      widget=gtk_label_new(NULL);
+      gchar id[16];
+      g_sprintf(id,"<b>%d</b>",i);
+      gtk_label_set_markup (GTK_LABEL(widget), id);
       gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
       col++;
-   
+
       widget=gtk_spin_button_new_with_range (0.0,28.0,1.0);
-      gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget),encoders[i].top_encoder_address_b);
+      gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget),encoders[i].bottom_encoder_address_a);
       gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
       col++;
-    }
-
-    if(i<(max_encoders-1)) {
+    
       widget=gtk_spin_button_new_with_range (0.0,28.0,1.0);
-      gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget),encoders[i].switch_address);
+      gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget),encoders[i].bottom_encoder_address_b);
       gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
       col++;
-    }
+    
+      if(controller==CONTROLLER2_V2 && i<(max_encoders-1)) {
+        widget=gtk_spin_button_new_with_range (0.0,28.0,1.0);
+        gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget),encoders[i].top_encoder_address_a);
+        gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
+        col++;
+   
+        widget=gtk_spin_button_new_with_range (0.0,28.0,1.0);
+        gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget),encoders[i].top_encoder_address_b);
+        gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
+        col++;
+      }
+
+      if(i<(max_encoders-1)) {
+        widget=gtk_spin_button_new_with_range (0.0,28.0,1.0);
+        gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget),encoders[i].switch_address);
+        gtk_grid_attach(GTK_GRID(grid),widget,col,row,1,1);
+        col++;
+      }
 
-    row++;
-    col=0;
-  }
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),grid,gtk_label_new("Encoders"));
+      row++;
+      col=0;
+    }
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),grid,gtk_label_new("Encoders"));
   }
 
 
@@ -334,8 +334,9 @@ void configure_gpio(GtkWidget *parent) {
 
 #ifdef LOCALCW
   //
-  // C2V2 uses *all* available GPIO lines, therefore we cannot do
-  // CW there. The special switch_address -1 means "do not use"
+  // Controller2 V2 uses *all* available GPIO lines, therefore we cannot
+  // present this tab.
+  // Accept "-1" as GPIO line, this means "do not use"
   //
   if(controller !=CONTROLLER2_V2) {
     char text[64];
index 335f66a431f62648cf53145bc9384b154ed6dd11..54d0e47389471da204c9f9feb2b0e822709374e0 100644 (file)
--- a/cw_menu.c
+++ b/cw_menu.c
@@ -33,9 +33,7 @@
 #include "receiver.h"
 #include "new_protocol.h"
 #include "old_protocol.h"
-#ifdef LOCALCW
 #include "iambic.h"
-#endif
 #include "ext.h"
 
 static GtkWidget *parent_window=NULL;
index 10c049b2535d7b4b49d8d7c87df733e2a5e50258..8811ed71530486717496cb0b2330caa4b2f29b7f 100644 (file)
 #ifdef SOAPYSDR
 #include "soapy_protocol.h"
 #endif
-#ifdef GPIO
 #include "actions.h"
 #include "gpio.h"
-#endif
 
 static GtkWidget *parent_window=NULL;
 
--- a/iambic.h
+++ b/iambic.h
@@ -1,6 +1,7 @@
 #ifndef _IAMBIC_H
 #define _IAMBIC_H
 
+#ifdef LOCALCW
 enum {
     CHECK = 0,
     STRAIGHT,
@@ -20,3 +21,4 @@ void keyer_close();
 int  keyer_init();
 
 #endif
+#endif
diff --git a/midi2.c b/midi2.c
index 7010074246585047804dffa1d845413c1dbca53c..7fa75d0e6672e7b0f99b40f114cc72877a46d295 100644 (file)
--- a/midi2.c
+++ b/midi2.c
@@ -327,10 +327,9 @@ int MIDIstartup(char *filename) {
     int chan;
     int t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12;
     int onoff, delay;
-    struct desc *desc,*dp;
+    struct desc *desc;
     enum MIDItype type;
     enum MIDIevent event;
-    int i;
     char c;
 
     MidiReleaseCommands();
diff --git a/midi3.c b/midi3.c
index eedf934cf9e1606f573fb09de74a1a633a38207b..0b5b7eea7d5497294d6de994880c8fac43be54db 100644 (file)
--- a/midi3.c
+++ b/midi3.c
@@ -444,6 +444,7 @@ int DoTheRestOfTheMIDI(void *data) {
         case MIDI_ACTION_DIV_COARSEPHASE:   // knob or wheel supported
         case MIDI_ACTION_DIV_FINEPHASE:     // knob or wheel supported
        case MIDI_ACTION_DIV_PHASE:             // knob or wheel supported
+            dnew=0.0;
             switch (type) {
               case MIDI_TYPE_KNOB:
                // coarse: change phase from -180 to 180
@@ -467,9 +468,7 @@ int DoTheRestOfTheMIDI(void *data) {
                }
                 break;
               default:
-                // do not change
-                // we should not come here anyway
-                dnew = 0.0;
+                // do nothing
                 break;
             }
             // dnew is the delta
@@ -804,10 +803,17 @@ int DoTheRestOfTheMIDI(void *data) {
             break;
        /////////////////////////////////////////////////////////// "RFGAIN"
         case MIDI_ACTION_RF_GAIN: // knob or wheel supported
-            if (type == MIDI_TYPE_KNOB) {
+            switch (type) {
+              case MIDI_TYPE_KNOB:
                 dnew=val;
-            } else  if (type == MIDI_TYPE_WHEEL) {
+                break;
+              case MIDI_TYPE_WHEEL:
                 dnew=adc[active_receiver->id].gain+val;
+                break;
+              default:
+                // Arriving here means there is an error somewhere else
+                dnew=0.0;
+                break;
             }
             if (dnew <   0.0) dnew =   0.0;
             if (dnew > 100.0) dnew = 100.0;
@@ -834,7 +840,8 @@ int DoTheRestOfTheMIDI(void *data) {
        /////////////////////////////////////////////////////////// "RITCLEAR"
        case MIDI_ACTION_RIT_CLEAR:       // only key supported
            // clear RIT value
-           vfo[active_receiver->id].rit = new;
+           vfo[active_receiver->id].rit = 0;
+           vfo[active_receiver->id].rit_enabled = 0;
             vfo_update();
            break;
        /////////////////////////////////////////////////////////// "RITSTEP"
index 14c0783c7de55bd5dd2dda5e55d7791aead388d1..3c5ff648118f083810d9ee3a9c08ee5cdafb8ebd 100644 (file)
@@ -50,11 +50,8 @@ enum {
 };
 
 static GtkWidget *parent_window=NULL;
-static GtkWidget *menu_b=NULL;
 static GtkWidget *dialog=NULL;
 
-static GtkWidget *midi_enable_b;
-
 static GtkListStore *store;
 static GtkWidget *view;
 static GtkWidget *scrolled_window=NULL;
@@ -64,8 +61,6 @@ static GtkTreeModel *model;
 static GtkTreeIter iter;
 struct desc *current_cmd;
 
-static GtkWidget *filename;
-
 static GtkWidget *newEvent;
 static GtkWidget *newChannel;
 static GtkWidget *newNote;
@@ -204,8 +199,6 @@ static void update_wheelparams(gpointer user_data) {
 }
 
 static void type_changed_cb(GtkWidget *widget, gpointer data) {
-  int i=0;
-  int j=0;
 
   // update actions available for the type
   gchar *type=gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(widget));
@@ -405,9 +398,7 @@ static void save_cb(GtkWidget *widget,gpointer user_data) {
   GtkWidget *save_dialog;
   GtkFileChooser *chooser;
   GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_SAVE;
-  gchar *filename;
   gint res;
-  struct desc *cmd;
 
   save_dialog = gtk_file_chooser_dialog_new ("Save File",
                                       GTK_WINDOW(dialog),
@@ -435,9 +426,7 @@ static void load_cb(GtkWidget *widget,gpointer user_data) {
   GtkWidget *load_dialog;
   GtkFileChooser *chooser;
   GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_OPEN;
-  gchar *filename;
   gint res;
-  struct desc *cmd;
 
   load_dialog = gtk_file_chooser_dialog_new ("Open MIDI File",
                                       GTK_WINDOW(dialog),
@@ -467,9 +456,7 @@ static void load_original_cb(GtkWidget *widget,gpointer user_data) {
   GtkWidget *load_dialog;
   GtkFileChooser *chooser;
   GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_OPEN;
-  gchar *filename;
   gint res;
-  struct desc *cmd;
 
   load_dialog = gtk_file_chooser_dialog_new ("Open LEGACY MIDI File",
                                       GTK_WINDOW(dialog),
@@ -679,6 +666,7 @@ static void update_cb(GtkButton *widget,gpointer user_data) {
   //g_print("%s: type=%s action=%s\n",__FUNCTION__,str_type,str_action);
 
   thisAction=MIDI_ACTION_NONE;
+  onoff=0;
   i=0;
   while(ActionTable[i].action!=MIDI_ACTION_LAST) {
     if(strcmp(ActionTable[i].str,str_action)==0) {
@@ -1224,8 +1212,6 @@ void midi_menu(GtkWidget *parent) {
 static int update(void *data) {
   int state=GPOINTER_TO_INT(data);
   gchar text[32];
-  gint i=1;
-  gint j;
 
   switch(state) {
     case UPDATE_NEW:
index 4c1b3ddf4ad44c6aee5cded222d63deddaa4a94c..91e4a5b431c7d059dc71ae854cf5db026b3840c0 100644 (file)
@@ -59,9 +59,7 @@
 #include "toolbar.h"
 #include "vox.h"
 #include "ext.h"
-#ifdef LOCALCW
 #include "iambic.h"
-#endif
 
 #define min(x,y) (x<y?x:y)
 
@@ -267,7 +265,7 @@ static pthread_mutex_t rx_spec_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t tx_spec_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t hi_prio_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t general_mutex = PTHREAD_MUTEX_INITIALIZER;
-static pthread_mutex_t p2_audio_mutex  = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t audio_mutex   = PTHREAD_MUTEX_INITIALIZER;
 
 static int local_ptt=0;
 
@@ -1995,7 +1993,7 @@ void new_protocol_cw_audio_samples(short left_audio_sample,short right_audio_sam
     //
     // Only process samples if transmitting in CW
     //
-    pthread_mutex_lock(&p2_audio_mutex);
+    pthread_mutex_lock(&audio_mutex);
     // insert the samples
     audiobuffer[audioindex++]=left_audio_sample>>8;
     audiobuffer[audioindex++]=left_audio_sample;
@@ -2017,7 +2015,7 @@ void new_protocol_cw_audio_samples(short left_audio_sample,short right_audio_sam
       audioindex=4;
       audiosequence++;
     }
-    pthread_mutex_unlock(&p2_audio_mutex);
+    pthread_mutex_unlock(&audio_mutex);
   }
 }
 
@@ -2030,7 +2028,7 @@ void new_protocol_audio_samples(RECEIVER *rx,short left_audio_sample,short right
   //
   if (isTransmitting() && (txmode==modeCWU || txmode==modeCWL)) return;
 
-  pthread_mutex_lock(&p2_audio_mutex);
+  pthread_mutex_lock(&audio_mutex);
   // insert the samples
   audiobuffer[audioindex++]=left_audio_sample>>8;
   audiobuffer[audioindex++]=left_audio_sample;
@@ -2054,7 +2052,7 @@ void new_protocol_audio_samples(RECEIVER *rx,short left_audio_sample,short right
     audioindex=4;
     audiosequence++;
   }
-  pthread_mutex_unlock(&p2_audio_mutex);
+  pthread_mutex_unlock(&audio_mutex);
 }
 
 void new_protocol_flush_iq_samples() {
index eece1fb667c1d10d7da0a48df7856ec76225b4ea..c4a0601b58c3fdfe192359b78107f2a683082ff0 100644 (file)
@@ -50,9 +50,7 @@
 #include "toolbar.h"
 #include "vfo.h"
 #include "ext.h"
-#ifdef LOCALCW
 #include "iambic.h"
-#endif
 #include "error_handler.h"
 
 #define min(x,y) (x<y?x:y)
index c68d1f718c169002594c52f86c9b66bd16e9d4cb..9ef19ccab6bf4462c279f08dbd52c395627be6c1 100644 (file)
 
 static PaStream *record_handle=NULL;
 
+
 int n_input_devices;
 AUDIO_DEVICE input_devices[MAX_AUDIO_DEVICES];
 int n_output_devices;
 AUDIO_DEVICE output_devices[MAX_AUDIO_DEVICES];
 
+static GMutex audio_mutex;
 int n_input_devices=0;
 int n_output_devices=0;
 
@@ -99,10 +101,11 @@ void audio_get_cards()
 
   PaError err;
 
+  g_mutex_init(&audio_mutex);
   err = Pa_Initialize();
   if( err != paNoError )
   {
-       fprintf(stderr, "PORTAUDIO ERROR: Pa_Initialize: %s\n", Pa_GetErrorText(err));
+       g_print("%s: init error %s\n", __FUNCTION__,Pa_GetErrorText(err));
        return;
   }
   numDevices = Pa_GetDeviceCount();
@@ -132,7 +135,7 @@ void audio_get_cards()
             input_devices[n_input_devices].index=i;
             n_input_devices++;
           }
-          fprintf(stderr,"PORTAUDIO INPUT DEVICE, No=%d, Name=%s\n", i, deviceInfo->name);
+          g_print("%s: INPUT DEVICE, No=%d, Name=%s\n", __FUNCTION__, i, deviceInfo->name);
        }
 
        outputParameters.device = i;
@@ -147,7 +150,7 @@ void audio_get_cards()
             output_devices[n_output_devices].index=i;
             n_output_devices++;
           }
-          fprintf(stderr,"PORTAUDIO OUTPUT DEVICE, No=%d, Name=%s\n", i, deviceInfo->name);
+          g_print("%s: OUTPUT DEVICE, No=%d, Name=%s\n", __FUNCTION__, i, deviceInfo->name);
         }
   }
 }
@@ -183,14 +186,16 @@ int audio_open_input()
        break;
     }
   }
-  fprintf(stderr,"audio_open_input: name=%s PADEV=%d\n",transmitter->microphone_name,padev);
+  g_print("%s: name=%s PADEV=%d\n", __FUNCTION__, transmitter->microphone_name, padev);
   //
-  // Should not occur, but possibly device name not found
+  // Device name possibly came from props file and device is no longer there
   //
   if (padev < 0) {
     return -1;
   }
 
+  g_mutex_lock(&audio_mutex);
+
   bzero( &inputParameters, sizeof( inputParameters ) ); //not necessary if you are filling in all the fields
   inputParameters.channelCount = 1;   // MONO
   inputParameters.device = padev;
@@ -202,23 +207,41 @@ int audio_open_input()
   err = Pa_OpenStream(&record_handle, &inputParameters, NULL, 48000.0, (unsigned long) MY_AUDIO_BUFFER_SIZE,
                       paNoFlag, pa_mic_cb, NULL);
   if (err != paNoError) {
-    fprintf(stderr, "PORTAUDIO ERROR: AOI open stream: %s\n",Pa_GetErrorText(err));
+    g_print("%s: open stream error %s\n", __FUNCTION__, Pa_GetErrorText(err));
+    record_handle=NULL;
+    g_mutex_unlock(&audio_mutex);
     return -1;
   }
 
   mic_ring_buffer=(float *) g_new(float,MY_RING_BUFFER_SIZE);
   mic_ring_outpt = mic_ring_inpt=0;
+
   if (mic_ring_buffer == NULL) {
+    Pa_CloseStream(record_handle);
+    record_handle=NULL;
+    g_print("%s: alloc buffer failed.\n", __FUNCTION__);
+    g_mutex_unlock(&audio_mutex);
     return -1;
   }
 
   err = Pa_StartStream(record_handle);
   if (err != paNoError) {
-    fprintf(stderr, "PORTAUDIO ERROR: AOI start stream:%s\n",Pa_GetErrorText(err));
+    g_print("%s: start stream error %s\n", __FUNCTION__, Pa_GetErrorText(err));
+    Pa_CloseStream(record_handle);
+    record_handle=NULL;
+    g_free(mic_ring_buffer);
+    mic_ring_buffer=NULL;
+    g_mutex_unlock(&audio_mutex);
     return -1;
   }
+
+  //
+  // Finished!
+  //
+  g_mutex_unlock(&audio_mutex);
   return 0;
 }
+
 //
 // PortAudio call-back function for Audio output
 //
@@ -230,28 +253,34 @@ int pa_out_cb(const void *inputBuffer, void *outputBuffer, unsigned long framesP
   float *out = (float *)outputBuffer;
   RECEIVER *rx = (RECEIVER *)userdata;
   int i, newpt;
-  float *buffer=rx->local_audio_buffer;
   int avail;  // only used for debug
 
   if (out == NULL) {
-    fprintf(stderr,"PortAudio error: bogus audio buffer in callback\n");
+    g_print("%s: bogus audio buffer in callback\n", __FUNCTION__);
     return paContinue;
   }
-  // DEBUG: report water mark
-  //avail = rx->local_audio_buffer_inpt - rx->local_audio_buffer_outpt;
-  //if (avail < 0) avail += MY_RING_BUFFER_SIZE;
-  //fprintf(stderr,"AVAIL=%d\n", avail);
-  newpt=rx->local_audio_buffer_outpt;
-  for (i=0; i< framesPerBuffer; i++) {
-    if (rx->local_audio_buffer_inpt == newpt) {
-      // Ring buffer empty, send zero sample
-      *out++ = 0.0;
-    } else {
-      *out++ = buffer[newpt++];
-      if (newpt >= MY_RING_BUFFER_SIZE) newpt=0;
-      rx->local_audio_buffer_outpt=newpt;
+  g_mutex_lock(&rx->local_audio_mutex);
+  if (rx->local_audio_buffer != NULL) {
+    //
+    // Mutex protection: if the buffer is non-NULL it cannot vanish
+    // util callback is completed
+    // DEBUG: report water mark
+    //avail = rx->local_audio_buffer_inpt - rx->local_audio_buffer_outpt;
+    //if (avail < 0) avail += MY_RING_BUFFER_SIZE;
+    //g_print("%s: AVAIL=%d\n", __FUNCTION__, avail);
+    newpt=rx->local_audio_buffer_outpt;
+    for (i=0; i< framesPerBuffer; i++) {
+      if (rx->local_audio_buffer_inpt == newpt) {
+        // Ring buffer empty, send zero sample
+        *out++ = 0.0;
+      } else {
+        *out++ = rx->local_audio_buffer[newpt++];
+        if (newpt >= MY_RING_BUFFER_SIZE) newpt=0;
+        rx->local_audio_buffer_outpt=newpt;
+      }
     }
   }
+  g_mutex_unlock(&rx->local_audio_mutex);
   return paContinue;
 }
 
@@ -265,44 +294,32 @@ int pa_mic_cb(const void *inputBuffer, void *outputBuffer, unsigned long framesP
 {
   float *in = (float *)inputBuffer;
   int i, newpt;
-  float sample;
 
   if (in == NULL) {
     // This should not happen, so we do not send silence etc.
-    fprintf(stderr,"PortAudio error: bogus audio buffer in callback\n");
+    g_print("%s: bogus audio buffer in callback\n", __FUNCTION__);
     return paContinue;
   }
-  //
-  // send the samples in the buffer
-  //
-  for (i=0; i<framesPerBuffer; i++) {
-    sample=in[i];
-    switch(protocol) {
-      case ORIGINAL_PROTOCOL:
-      case NEW_PROTOCOL:
-#ifdef SOAPYSDR
-      case SOAPYSDR_PROTOCOL:
-#endif
-       //
-       // put sample into ring buffer
-       //
-       if (mic_ring_buffer != NULL) {
-          // the "existence" of the ring buffer is now guaranteed for 1 msec,
-          // see audio_close_input(),
-         newpt=mic_ring_inpt +1;
-         if (newpt == MY_RING_BUFFER_SIZE) newpt=0;
-         if (newpt != mic_ring_outpt) {
-           // buffer space available, do the write
-           mic_ring_buffer[mic_ring_inpt]=sample;
-           // atomic update of mic_ring_inpt
-           mic_ring_inpt=newpt;
-         }
-       }
-       break;
-      default:
-       break;
+  g_mutex_lock(&audio_mutex);
+  if (mic_ring_buffer != NULL) {
+    //
+    // mutex protected: ring buffer cannot vanish
+    //
+    for (i=0; i<framesPerBuffer; i++) {
+      //
+      // put sample into ring buffer
+      //
+      newpt=mic_ring_inpt +1;
+      if (newpt == MY_RING_BUFFER_SIZE) newpt=0;
+      if (newpt != mic_ring_outpt) {
+        // buffer space available, do the write
+        mic_ring_buffer[mic_ring_inpt]=in[i];
+        // atomic update of mic_ring_inpt
+        mic_ring_inpt=newpt;
+      }
     }
   }
+  g_mutex_unlock(&audio_mutex);
   return paContinue;
 }
 
@@ -313,18 +330,22 @@ int pa_mic_cb(const void *inputBuffer, void *outputBuffer, unsigned long framesP
 float audio_get_next_mic_sample() {
   int newpt;
   float sample;
+  g_mutex_lock(&audio_mutex);
+  //
+  // mutex protected (for every single sample!):
+  // ring buffer cannot vanish while being processed here
+  //
   if ((mic_ring_buffer == NULL) || (mic_ring_outpt == mic_ring_inpt)) {
     // no buffer, or nothing in buffer: insert silence
     sample=0.0;
   } else {
-    // the "existence" of the ring buffer is now guaranteed for 1 msec,
-    // see audio_close_input(),
     newpt = mic_ring_outpt+1;
     if (newpt == MY_RING_BUFFER_SIZE) newpt=0;
     sample=mic_ring_buffer[mic_ring_outpt];
     // atomic update of read pointer
     mic_ring_outpt=newpt;
   }
+  g_mutex_unlock(&audio_mutex);
   return sample;
 }
 
@@ -354,15 +375,16 @@ int audio_open_output(RECEIVER *rx)
         break;
     }
   }
-  fprintf(stderr,"audio_open_output: name=%s PADEV=%d\n",rx->audio_name,padev);
+  g_print("%s: name=%s PADEV=%d\n", __FUNCTION__, rx->audio_name, padev); 
   //
-  // Should not occur, but possibly device name not found
+  // Device name possibly came from props file and device is no longer there
   //
   if (padev < 0) {
     return -1;
   }
 
   g_mutex_lock(&rx->local_audio_mutex);
+
   bzero( &outputParameters, sizeof( outputParameters ) ); //not necessary if you are filling in all the fields
   outputParameters.channelCount = 1;   // Always MONO
   outputParameters.device = padev;
@@ -372,6 +394,15 @@ int audio_open_output(RECEIVER *rx)
   outputParameters.suggestedLatency = 0.0; //Pa_GetDeviceInfo(padev)->defaultLowOutputLatency ;
   outputParameters.hostApiSpecificStreamInfo = NULL; //See you specific host's API docs for info on using this field
 
+  err = Pa_OpenStream(&(rx->playstream), NULL, &outputParameters, 48000.0, (unsigned long) MY_AUDIO_BUFFER_SIZE,
+                      paNoFlag, pa_out_cb, rx);
+  if (err != paNoError) {
+    g_print("%s: open stream error %s\n", __FUNCTION__, Pa_GetErrorText(err));
+    rx->playstream = NULL;
+    g_mutex_unlock(&rx->local_audio_mutex);
+    return -1;
+  }
+
   //
   // This is now a ring buffer much larger than a single audio buffer
   //
@@ -380,26 +411,27 @@ int audio_open_output(RECEIVER *rx)
   rx->local_audio_buffer_outpt=0;
   rx->local_audio_cw=0;
 
-  err = Pa_OpenStream(&(rx->playback_handle), NULL, &outputParameters, 48000.0, (unsigned long) MY_AUDIO_BUFFER_SIZE,
-                      paNoFlag, pa_out_cb, rx);
-  if (err != paNoError || rx->local_audio_buffer == NULL) {
-    fprintf(stderr,"PORTAUDIO ERROR: out open stream: %s\n",Pa_GetErrorText(err));
-    rx->playback_handle = NULL;
-    if (rx->local_audio_buffer) g_free(rx->local_audio_buffer);
-    rx->local_audio_buffer = NULL;
+  if (rx->local_audio_buffer == NULL) {
+    g_print("%s: allocate buffer failed\n", __FUNCTION__);
+    Pa_CloseStream(rx->playstream);
+    rx->playstream=NULL;
     g_mutex_unlock(&rx->local_audio_mutex);
     return -1;
   }
 
-  err = Pa_StartStream(rx->playback_handle);
+  err = Pa_StartStream(rx->playstream);
   if (err != paNoError) {
-    fprintf(stderr,"PORTAUDIO ERROR: out start stream:%s\n",Pa_GetErrorText(err));
-    rx->playback_handle=NULL;
-    if (rx->local_audio_buffer) g_free(rx->local_audio_buffer);
+    g_print("%s: error starting stream:%s\n",__FUNCTION__,Pa_GetErrorText(err));
+    Pa_CloseStream(rx->playstream);
+    rx->playstream=NULL;
+    g_free(rx->local_audio_buffer);
     rx->local_audio_buffer = NULL;
     g_mutex_unlock(&rx->local_audio_mutex);
     return -1;
   }
+  //
+  // Finished!
+  //
   g_mutex_unlock(&rx->local_audio_mutex);
   return 0;
 }
@@ -412,34 +444,25 @@ int audio_open_output(RECEIVER *rx)
 void audio_close_input()
 {
   PaError err;
-  void *p;
 
-  fprintf(stderr,"AudioCloseInput: %s\n", transmitter->microphone_name);
+  g_print("%s: micname=%s\n", __FUNCTION__,transmitter->microphone_name);
 
+  g_mutex_lock(&audio_mutex);
   if(record_handle!=NULL) {
     err = Pa_StopStream(record_handle);
     if (err != paNoError) {
-      fprintf(stderr,"PORTAUDIO ERROR: in stop stream: %s\n",Pa_GetErrorText(err));
+      g_print("%s: error stopping stream: %s\n",__FUNCTION__,Pa_GetErrorText(err));
     }
     err = Pa_CloseStream(record_handle);
     if (err != paNoError) {
-      fprintf(stderr,"PORTAUDIO ERROR: in close stream: %s\n",Pa_GetErrorText(err));
+      g_print("%s: %s\n",__FUNCTION__,Pa_GetErrorText(err));
     }
     record_handle=NULL;
   }
-  //
-  // We do not want to do a mutex lock/unlock for every single mic sample
-  // accessed. Since only the ring buffer is maintained by the functions
-  // audio_get_next_mic_sample() and in the "mic callback" function,
-  // it is more than enough to wait 2 msec after setting mic_ring_buffer to NULL
-  // before actually releasing the storage.
-  //
   if (mic_ring_buffer != NULL) {
-    p=mic_ring_buffer;
-    mic_ring_buffer=NULL;
-    usleep(2);
-    g_free(p);
+    g_free(mic_ring_buffer);
   }
+  g_mutex_unlock(&audio_mutex);
 }
 
 //
@@ -450,7 +473,7 @@ void audio_close_input()
 void audio_close_output(RECEIVER *rx) {
   PaError err;
 
-  fprintf(stderr,"AudioCloseOutput: %s\n", rx->audio_name);
+  g_print("%s: device=%sn", __FUNCTION__,rx->audio_name);
 
   g_mutex_lock(&rx->local_audio_mutex);
   if(rx->local_audio_buffer!=NULL) {
@@ -458,16 +481,16 @@ void audio_close_output(RECEIVER *rx) {
     rx->local_audio_buffer=NULL;
   }
 
-  if(rx->playback_handle!=NULL) {
-    err = Pa_StopStream(rx->playback_handle);
+  if(rx->playstream!=NULL) {
+    err = Pa_StopStream(rx->playstream);
     if (err != paNoError) {
-      fprintf(stderr,"PORTAUDIO ERROR: out stop stream: %s\n",Pa_GetErrorText(err));
+      g_print("%s: stop stream error %s\n", __FUNCTION__, Pa_GetErrorText(err));
     }
-    err = Pa_CloseStream(rx->playback_handle);
+    err = Pa_CloseStream(rx->playstream);
     if (err != paNoError) {
-      fprintf(stderr,"PORTAUDIO ERROR: out close stream: %s\n",Pa_GetErrorText(err));
+      g_print("%s: close stream error %s\n",__FUNCTION__,Pa_GetErrorText(err));
     }
-    rx->playback_handle=NULL;
+    rx->playstream=NULL;
   }
   g_mutex_unlock(&rx->local_audio_mutex);
 }
@@ -504,7 +527,7 @@ int audio_write (RECEIVER *rx, float left, float right)
   }
 
   g_mutex_lock(&rx->local_audio_mutex);
-  if (rx->playback_handle != NULL && buffer != NULL) {
+  if (rx->playstream != NULL && buffer != NULL) {
     if (rx->local_audio_cw == 1) {
       //
       // We come from a TX->RX transition:
@@ -578,21 +601,20 @@ int audio_write (RECEIVER *rx, float left, float right)
 // Thus we have an active latency management.
 //
 int cw_audio_write(RECEIVER *rx, float sample) {
-  float *buffer = rx->local_audio_buffer;
   int oldpt, newpt;
   static int count=0;
   int avail;
   int adjust;
 
   g_mutex_lock(&rx->local_audio_mutex);
-  if (rx->playback_handle != NULL && buffer != NULL) {
+  if (rx->playstream != NULL && rx->local_audio_buffer != NULL) {
     if (rx->local_audio_cw == 0 && cw_keyer_sidetone_volume > 0) {
       //
       // First time producing CW audio after RX/TX transition:
       // empty audio buffer and insert 512 samples of silence
       //
       rx->local_audio_cw=1;
-      bzero(buffer, 512*sizeof(float));
+      bzero(rx->local_audio_buffer, 512*sizeof(float));
       rx->local_audio_buffer_inpt=512;
       rx->local_audio_buffer_outpt=0;
       count=0;
@@ -625,7 +647,7 @@ int cw_audio_write(RECEIVER *rx, float sample) {
           //
           // buffer space available
           //
-          buffer[oldpt] = sample;
+          rx->local_audio_buffer[oldpt] = sample;
           rx->local_audio_buffer_inpt=newpt;
         }
         break;
@@ -635,16 +657,16 @@ int cw_audio_write(RECEIVER *rx, float sample) {
         // insert two samples of silence. No check on "buffer full" needed
         //
         oldpt=rx->local_audio_buffer_inpt;
-        buffer[oldpt++]=0.0;
+        rx->local_audio_buffer[oldpt++]=0.0;
         if (oldpt == MY_RING_BUFFER_SIZE) oldpt=0;
-        buffer[oldpt++]=0.0;
+        rx->local_audio_buffer[oldpt++]=0.0;
         if (oldpt == MY_RING_BUFFER_SIZE) oldpt=0;
         rx->local_audio_buffer_inpt=oldpt;
         break;
       case 2:
         //
         // buffer becomes too full, and we just saw
-        // 16 samples of silence: just skip it
+        // 16 samples of silence: just skip the last "silent" sample
         //
         break;
     }
index 2435907e6c13ff12805b43a67690d2de6b2fd0e2..3ea44574f5bce70b9e94d617fd39eddb49259896 100644 (file)
@@ -21,7 +21,7 @@ AUDIO_DEVICE output_devices[MAX_AUDIO_DEVICES];
 
 //
 // Ring buffer for "local microphone" samples
-// NOTE: lead large buffer for some "loopback" devices which produce
+// NOTE: need large buffer for some "loopback" devices which produce
 //       samples in large chunks if fed from digimode programs.
 //
 #define MICRINGLEN 6000
@@ -35,12 +35,12 @@ static pa_operation *op;
 static pa_context *pa_ctx;
 static pa_simple* microphone_stream;
 static gint local_microphone_buffer_offset;
-static float *local_microphone_buffer;
-static GThread *mic_read_thread_id;
+static float *local_microphone_buffer=NULL;
+static GThread *mic_read_thread_id=0;
 static gboolean running;
 
 gint local_microphone_buffer_size=720;
-GMutex audio_mutex;
+static GMutex audio_mutex;
 
 static void source_list_cb(pa_context *context,const pa_source_info *s,int eol,void *data) {
   int i;
@@ -122,7 +122,7 @@ void audio_get_cards() {
   g_mutex_lock(&audio_mutex);
   main_loop=pa_glib_mainloop_new(NULL);
   main_loop_api=pa_glib_mainloop_get_api(main_loop);
-  pa_ctx=pa_context_new(main_loop_api,"linhpsdr");
+  pa_ctx=pa_context_new(main_loop_api,"piHPSDR");
   pa_context_connect(pa_ctx,NULL,0,NULL);
   pa_context_set_state_callback(pa_ctx, state_cb, NULL);
 }
@@ -175,50 +175,34 @@ static void *mic_read_thread(gpointer arg) {
 
   g_print("%s: running=%d\n",__FUNCTION__,running);
   while(running) {
-    g_mutex_lock(&audio_mutex);
-    if(local_microphone_buffer==NULL) {
-      running=0;
-    } else {
-      rc=pa_simple_read(microphone_stream,
+    //
+    // It is guaranteed that local_microphone_buffer, mic_ring_buffer, and microphone_stream
+    // will not be destroyed until this thread has terminated (and waited for via thread joining)
+    //
+    rc=pa_simple_read(microphone_stream,
             local_microphone_buffer,
             local_microphone_buffer_size*sizeof(float),
             &err);
-      if(rc<0) {
-        running=0;
-        g_print("%s: returned %d error=%d (%s)\n",__FUNCTION__,rc,err,pa_strerror(err));
-      } else {
-       gint newpt;
-       for(gint i=0;i<local_microphone_buffer_size;i++) {
-          gfloat sample=local_microphone_buffer[i];
-          switch(protocol) {
-            case ORIGINAL_PROTOCOL:
-            case NEW_PROTOCOL:
-#ifdef SOAPYSDR
-            case SOAPYSDR_PROTOCOL:
-#endif
-              //
-              // put sample into ring buffer
-              //
-              if (mic_ring_buffer != NULL) {
-                // the "existence" of the ring buffer is now guaranteed for 1 msec,
-                // see audio_close_input().
-                newpt=mic_ring_write_pt +1;
-                if (newpt == MICRINGLEN) newpt=0;
-                if (newpt != mic_ring_read_pt) {
-                  // buffer space available, do the write
-                  mic_ring_buffer[mic_ring_write_pt]=sample;
-                  // atomic update of mic_ring_write_pt
-                  mic_ring_write_pt=newpt;
-                }
-              }
-              break;
-            default:
-              break;
-          }
+    if(rc<0) {
+      running=FALSE;
+      g_print("%s: returned %d error=%d (%s)\n",__FUNCTION__,rc,err,pa_strerror(err));
+    } else {
+      int newpt;
+      for(gint i=0;i<local_microphone_buffer_size;i++) {
+        gfloat sample=local_microphone_buffer[i];
+        //
+        // put sample into ring buffer
+        //
+        newpt=mic_ring_write_pt +1;
+        if (newpt == MICRINGLEN) newpt=0;
+        if (newpt != mic_ring_read_pt) {
+          // buffer space available, do the write
+          mic_ring_buffer[mic_ring_write_pt]=sample;
+          // atomic update of mic_ring_write_pt
+          mic_ring_write_pt=newpt;
         }
       }
     }
-    g_mutex_unlock(&audio_mutex);
   }
   g_print("%s: exit\n",__FUNCTION__);
 }
@@ -267,6 +251,7 @@ int audio_open_input() {
     mic_ring_buffer=(float *) g_new(float,MICRINGLEN);
     mic_ring_read_pt = mic_ring_write_pt=0;
     if (mic_ring_buffer == NULL) {
+      g_mutex_unlock(&audio_mutex);
       audio_close_input();
       return -1;
     }
@@ -303,13 +288,30 @@ void audio_close_output(RECEIVER *rx) {
 }
 
 void audio_close_input() {
+  running=FALSE;
   g_mutex_lock(&audio_mutex);
+
+  if(mic_read_thread_id!=NULL) {
+g_print("audio_close_input: wait for thread to complete\n");
+    //
+    // wait for the mic read thread to terminate,
+    // then destroy the stream and the buffers
+    // This way, the buffers cannot "vanish" in the mic read thread
+    //
+    g_thread_join(mic_read_thread_id);
+    mic_read_thread_id=NULL;
+  }
   if(microphone_stream!=NULL) {
     pa_simple_free(microphone_stream);
     microphone_stream=NULL;
+  }
+  if (local_microphone_buffer != NULL) {
     g_free(local_microphone_buffer);
     local_microphone_buffer=NULL;
   }
+  if (mic_ring_buffer != NULL) {
+    g_free(mic_ring_buffer);
+  }
   g_mutex_unlock(&audio_mutex);
 }
 
@@ -320,20 +322,18 @@ void audio_close_input() {
 float audio_get_next_mic_sample() {
   int newpt;
   float sample;
-
+  g_mutex_lock(&audio_mutex);
   if ((mic_ring_buffer == NULL) || (mic_ring_read_pt == mic_ring_write_pt)) {
     // no buffer, or nothing in buffer: insert silence
-    g_print("%s: no samples\n",__FUNCTION__);
     sample=0.0;
   } else {
-    // the "existence" of the ring buffer is now guaranteed for 1 msec,
-    // see audio_close_input(),
     newpt = mic_ring_read_pt+1;
     if (newpt == MICRINGLEN) newpt=0;
     sample=mic_ring_buffer[mic_ring_read_pt];
     // atomic update of read pointer
     mic_ring_read_pt=newpt;
   }
+  g_mutex_unlock(&audio_mutex);
   return sample;
 }
 
@@ -343,23 +343,21 @@ int cw_audio_write(RECEIVER *rx,float sample) {
   int err;
 
   g_mutex_lock(&rx->local_audio_mutex);
-  if(rx->local_audio_buffer==NULL) {
-    rx->local_audio_buffer_offset=0;
-    rx->local_audio_buffer=g_new0(float,2*rx->local_audio_buffer_size);
-  }
 
-  rx->local_audio_buffer[rx->local_audio_buffer_offset*2]=sample;
-  rx->local_audio_buffer[(rx->local_audio_buffer_offset*2)+1]=sample;
-  rx->local_audio_buffer_offset++;
-  if(rx->local_audio_buffer_offset>=rx->local_audio_buffer_size) {
-    rc=pa_simple_write(rx->playstream,
-                       rx->local_audio_buffer,
-                       rx->local_audio_buffer_size*sizeof(float)*2,
-                       &err);
-    if(rc!=0) {
-      fprintf(stderr,"audio_write failed err=%d\n",err);
+  if (rx->playstream != NULL && rx->local_audio_buffer != NULL) {  // Note this test must be done "mutex-protected"
+    rx->local_audio_buffer[rx->local_audio_buffer_offset*2]=sample;
+    rx->local_audio_buffer[(rx->local_audio_buffer_offset*2)+1]=sample;
+    rx->local_audio_buffer_offset++;
+    if(rx->local_audio_buffer_offset>=rx->local_audio_buffer_size) {
+      rc=pa_simple_write(rx->playstream,
+                         rx->local_audio_buffer,
+                         rx->local_audio_buffer_size*sizeof(float)*2,
+                         &err);
+      if(rc!=0) {
+        fprintf(stderr,"cw_audio_write failed err=%d\n",err);
+      }
+      rx->local_audio_buffer_offset=0;
     }
-    rx->local_audio_buffer_offset=0;
   }
   g_mutex_unlock(&rx->local_audio_mutex);
 
@@ -372,23 +370,21 @@ int audio_write(RECEIVER *rx,float left_sample,float right_sample) {
   int err;
 
   g_mutex_lock(&rx->local_audio_mutex);
-  if(rx->local_audio_buffer==NULL) {
-    rx->local_audio_buffer_offset=0;
-    rx->local_audio_buffer=g_new0(float,2*rx->local_audio_buffer_size);
-  }
 
-  rx->local_audio_buffer[rx->local_audio_buffer_offset*2]=left_sample;
-  rx->local_audio_buffer[(rx->local_audio_buffer_offset*2)+1]=right_sample;
-  rx->local_audio_buffer_offset++;
-  if(rx->local_audio_buffer_offset>=rx->local_audio_buffer_size) {
-    rc=pa_simple_write(rx->playstream,
-                       rx->local_audio_buffer,
-                       rx->local_audio_buffer_size*sizeof(float)*2,
-                       &err);
-    if(rc!=0) {
-      fprintf(stderr,"audio_write failed err=%d\n",err);
+  if (rx->playstream != NULL && rx->local_audio_buffer != NULL) {  // Note this test must be done "mutex-protected"
+    rx->local_audio_buffer[rx->local_audio_buffer_offset*2]=left_sample;
+    rx->local_audio_buffer[(rx->local_audio_buffer_offset*2)+1]=right_sample;
+    rx->local_audio_buffer_offset++;
+    if(rx->local_audio_buffer_offset>=rx->local_audio_buffer_size) {
+      rc=pa_simple_write(rx->playstream,
+                         rx->local_audio_buffer,
+                         rx->local_audio_buffer_size*sizeof(float)*2,
+                         &err);
+      if(rc!=0) {
+        fprintf(stderr,"audio_write failed err=%d\n",err);
+      }
+      rx->local_audio_buffer_offset=0;
     }
-    rx->local_audio_buffer_offset=0;
   }
   g_mutex_unlock(&rx->local_audio_mutex);
 
index 28730bf72007dcf136499e352c4d9d095b2cf362..b4b6199edc18a4368dcc86af6c62ebcc9b500b27 100644 (file)
@@ -834,7 +834,7 @@ g_print("%s: panadapter height=%d y=%d %p\n",__FUNCTION__,height,y,rx->panadapte
 
   if(rx->display_waterfall) {
     waterfall_init(rx,rx->width,height);
-g_print("%ss: waterfall height=%d y=%d %p\n",__FUNCTION__,height,y,rx->waterfall);
+g_print("%s: waterfall height=%d y=%d %p\n",__FUNCTION__,height,y,rx->waterfall);
     g_object_weak_ref(G_OBJECT(rx->waterfall),receiver_weak_notify,(gpointer)rx);
     gtk_fixed_put(GTK_FIXED(rx->panel),rx->waterfall,0,y);
   }
@@ -899,7 +899,6 @@ g_print("%s: id=%d buffer_size=%d\n",__FUNCTION__,id,buffer_size);
   rx->panadapter_step=20;
 
   rx->volume=5.0;
-  //rx->rf_gain=50.0;
 
   rx->squelch_enable=0;
   rx->squelch=0;
@@ -928,12 +927,6 @@ g_print("%s: id=%d buffer_size=%d\n",__FUNCTION__,id,buffer_size);
   rx->agc_slope=35.0;
   rx->agc_hang_threshold=0.0;
   
-#if defined(ALSA) || defined(PORTAUDIO)
-  rx->playback_handle=NULL;
-#endif
-#ifdef PULSEAUDIO
-  rx->playstream=NULL;
-#endif
   rx->local_audio_buffer=NULL;
   rx->local_audio_buffer_size=2048;
   rx->local_audio=0;
@@ -1041,7 +1034,6 @@ g_print("%s: id=%d sample_rate=%d\n",__FUNCTION__,rx->id, rx->sample_rate);
   rx->waterfall_automatic=1;
 
   rx->volume=0.1;
-  //rx->rf_gain=50.0;
 
   rx->dither=0;
   rx->random=0;
@@ -1068,7 +1060,6 @@ g_print("%s: id=%d sample_rate=%d\n",__FUNCTION__,rx->id, rx->sample_rate);
   rx->agc_slope=35.0;
   rx->agc_hang_threshold=0.0;
   
-  //rx->playback_handle=NULL;
   rx->local_audio=0;
   g_mutex_init(&rx->local_audio_mutex);
   rx->local_audio_buffer=NULL;
index 747f2c2fef2f139ae854f26622ede1c01d504c31..93aee97d869957c6c3d94a2fb77fdb0fc0c1eace 100644 (file)
@@ -121,7 +121,7 @@ typedef struct _receiver {
   gint audio_device;
   gchar *audio_name;
 #ifdef PORTAUDIO
-  PaStream *playback_handle;
+  PaStream *playstream;
   gint local_audio_buffer_inpt;    // pointer in audio ring-buffer
   gint local_audio_buffer_outpt;   // pointer in audio ring-buffer
   float *local_audio_buffer;
index 7aadb346bca7482da16e26b13f24bafbf1775cab..d29aa9df44105ed5719210c42a70145ec7605e3c 100644 (file)
--- a/rigctl.c
+++ b/rigctl.c
@@ -53,9 +53,7 @@
 #include "rigctl_menu.h"
 #include "noise_menu.h"
 #include "new_protocol.h"
-#ifdef LOCALCW
 #include "iambic.h"              // declare keyer_update()
-#endif
 #include <math.h>
 
 #define NEW_PARSER
@@ -801,7 +799,7 @@ gboolean parse_extended_cmd (char *command,CLIENT *client) {
           if(command[4]==';') {
             // read the step size
             int i=0;
-            for(i=0;i<=14;i++) {
+            for(i=0;i<STEPS;i++) {
               if(steps[i]==step) break;
             }
             if(i<=14) {
@@ -812,7 +810,7 @@ gboolean parse_extended_cmd (char *command,CLIENT *client) {
           } else if(command[6]==';') {
             // set the step size
             int i=atoi(&command[4]) ;
-            if(i>=0 && i<=14) {
+            if(i>=0 && i<STEPS) {
               step=steps[i];
               vfo_update();
             }
@@ -824,7 +822,7 @@ gboolean parse_extended_cmd (char *command,CLIENT *client) {
           if(command[6]==';') {
             int step_index=atoi(&command[4]);
             long long hz=0;
-            if(step_index>=0 && step_index<=14) {
+            if(step_index>=0 && step_index<STEPS) {
               hz=(long long)steps[step_index];
             }
             if(hz!=0LL) {
@@ -897,7 +895,7 @@ gboolean parse_extended_cmd (char *command,CLIENT *client) {
           if(command[6]==';') {
             int step_index=atoi(&command[4]);
             long long hz=0;
-            if(step_index>=0 && step_index<=14) {
+            if(step_index>=0 && step_index<STEPS) {
               hz=(long long)steps[step_index];
             }
             if(hz!=0LL) {
@@ -961,7 +959,7 @@ gboolean parse_extended_cmd (char *command,CLIENT *client) {
           if(command[6]==';') {
             int step_index=atoi(&command[4]);
             long long hz=0;
-            if(step_index>=0 && step_index<=14) {
+            if(step_index>=0 && step_index<STEPS) {
               hz=(long long)steps[step_index];
             }
             if(hz!=0LL) {
@@ -976,7 +974,7 @@ gboolean parse_extended_cmd (char *command,CLIENT *client) {
           if(command[6]==';') {
             int step_index=atoi(&command[4]);
             long long hz=0;
-            if(step_index>=0 && step_index<=14) {
+            if(step_index>=0 && step_index<STEPS) {
               hz=(long long)steps[step_index];
             }
             if(hz!=0LL) {
index fdac59b4a475960551aca7f0a126c2a2aefeb5f5..b2d80c5b3379795f3f0db53adb88ad6b4d99d0ee 100644 (file)
@@ -38,7 +38,6 @@
 #include "rx_panadapter.h"
 #include "vfo.h"
 #include "mode.h"
-#include "actions.h"
 #ifdef GPIO
 #include "gpio.h"
 #endif
@@ -46,7 +45,8 @@
 #include "client_server.h"
 #endif
 
-#define LINE_WIDTH 0.5
+#define LINE_THIN  0.5
+#define LINE_THICK 1.0
 
 //static float panadapter_max=-60.0;
 //static float panadapter_min=-160.0;
@@ -139,7 +139,7 @@ void rx_panadapter_update(RECEIVER *rx) {
   //clear_panadater_surface();
   cairo_t *cr;
   cr = cairo_create (rx->panadapter_surface);
-  cairo_set_line_width(cr, LINE_WIDTH);
+  cairo_set_line_width(cr, LINE_THIN);
   cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
   cairo_rectangle(cr,0,0,display_width,display_height);
   cairo_fill(cr);
@@ -222,6 +222,7 @@ void rx_panadapter_update(RECEIVER *rx) {
     cw_frequency=filter_left+((filter_right-filter_left)/2.0);
     cairo_move_to(cr,cw_frequency,10.0);
     cairo_line_to(cr,cw_frequency,(double)display_height);
+    cairo_set_line_width(cr, LINE_THICK);
     cairo_stroke(cr);
   }
 
@@ -233,7 +234,7 @@ void rx_panadapter_update(RECEIVER *rx) {
   }
 
   double dbm_per_line=(double)display_height/((double)rx->panadapter_high-(double)rx->panadapter_low);
-  cairo_set_line_width(cr, LINE_WIDTH);
+  cairo_set_line_width(cr, LINE_THIN);
   cairo_select_font_face(cr, DISPLAY_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
   cairo_set_font_size(cr, DISPLAY_FONT_SIZE2);
   char v[32];
@@ -250,6 +251,7 @@ void rx_panadapter_update(RECEIVER *rx) {
       cairo_show_text(cr, v);
     }
   }
+  cairo_set_line_width(cr, LINE_THIN);
   cairo_stroke(cr);
 
   // plot frequency markers
@@ -366,7 +368,6 @@ void rx_panadapter_update(RECEIVER *rx) {
   }
 
   f = ((min_display/divisor)*divisor)+divisor;
-
   cairo_select_font_face(cr, DISPLAY_FONT,
                             CAIRO_FONT_SLANT_NORMAL,
                             CAIRO_FONT_WEIGHT_BOLD);
@@ -382,23 +383,26 @@ void rx_panadapter_update(RECEIVER *rx) {
     cairo_show_text(cr, v);
     f+=divisor;
   }
+  cairo_set_line_width(cr, LINE_THIN);
   cairo_stroke(cr);
 
   if(vfoband!=band60) {
     // band edges
     if(band->frequencyMin!=0LL) {
       cairo_set_source_rgb (cr, 1.0, 0.0, 0.0);
-      cairo_set_line_width(cr, LINE_WIDTH);
+      cairo_set_line_width(cr, LINE_THICK);
       if((min_display<band->frequencyMin)&&(max_display>band->frequencyMin)) {
         i=(band->frequencyMin-min_display)/(long long)HzPerPixel;
         cairo_move_to(cr,(double)i,0.0);
         cairo_line_to(cr,(double)i,(double)display_height);
+        cairo_set_line_width(cr, LINE_THICK);
         cairo_stroke(cr);
       }
       if((min_display<band->frequencyMax)&&(max_display>band->frequencyMax)) {
         i=(band->frequencyMax-min_display)/(long long)HzPerPixel;
         cairo_move_to(cr,(double)i,0.0);
         cairo_line_to(cr,(double)i,(double)display_height);
+        cairo_set_line_width(cr, LINE_THICK);
         cairo_stroke(cr);
       }
     }
@@ -421,6 +425,7 @@ void rx_panadapter_update(RECEIVER *rx) {
 
   // agc
   if(rx->agc!=AGC_OFF) {
+    cairo_set_line_width(cr, LINE_THICK);
     double knee_y=rx->agc_thresh+(double)adc[rx->adc].attenuation;
     if (filter_board == ALEX && rx->adc == 0) knee_y += (double)(10*rx->alex_attenuation);
     knee_y = floor((rx->panadapter_high - knee_y)
@@ -444,6 +449,7 @@ void rx_panadapter_update(RECEIVER *rx) {
       cairo_fill(cr);
       cairo_move_to(cr,40.0,hang_y);
       cairo_line_to(cr,(double)display_width-40.0,hang_y);
+      cairo_set_line_width(cr, LINE_THICK);
       cairo_stroke(cr);
       cairo_move_to(cr,48.0,hang_y);
       cairo_show_text(cr, "-H");
@@ -459,6 +465,7 @@ void rx_panadapter_update(RECEIVER *rx) {
     cairo_fill(cr);
     cairo_move_to(cr,40.0,knee_y);
     cairo_line_to(cr,(double)display_width-40.0,knee_y);
+    cairo_set_line_width(cr, LINE_THICK);
     cairo_stroke(cr);
     cairo_move_to(cr,48.0,knee_y);
     cairo_show_text(cr, "-G");
@@ -471,9 +478,9 @@ void rx_panadapter_update(RECEIVER *rx) {
   } else {
     cairo_set_source_rgb (cr, 0.5, 0.0, 0.0);
   }
-  cairo_set_line_width(cr, LINE_WIDTH);
   cairo_move_to(cr,vfofreq+(offset/HzPerPixel),0.0);
   cairo_line_to(cr,vfofreq+(offset/HzPerPixel),(double)display_height);
+  cairo_set_line_width(cr, LINE_THIN);
   cairo_stroke(cr);
 
   // signal
@@ -501,7 +508,6 @@ void rx_panadapter_update(RECEIVER *rx) {
   }
 #ifdef SOAPYSDR
   if(protocol==SOAPYSDR_PROTOCOL) {
-    //s1-=rx->rf_gain;
     s1-=adc[rx->adc].gain;
   }
 #endif
@@ -523,7 +529,6 @@ void rx_panadapter_update(RECEIVER *rx) {
     }
 #ifdef SOAPYSDR
     if(protocol==SOAPYSDR_PROTOCOL) {
-      //s2-=rx->rf_gain;
       s2-=adc[rx->adc].gain;
     }
 #endif
@@ -533,7 +538,8 @@ void rx_panadapter_update(RECEIVER *rx) {
     cairo_line_to(cr, (double)i, s2);
   }
 
-  cairo_pattern_t *gradient=NULL;
+  cairo_pattern_t *gradient;
+  gradient=NULL;
   if(display_gradient) {
     gradient = cairo_pattern_create_linear(0.0, display_height, 0.0, 0.0);
     // calculate where S9 is
@@ -572,12 +578,12 @@ void rx_panadapter_update(RECEIVER *rx) {
   if(display_filled) {
     cairo_close_path (cr);
     cairo_fill_preserve (cr);
-    cairo_set_line_width(cr, LINE_WIDTH);
+    cairo_set_line_width(cr, LINE_THIN);
   } else {
     //
-    // if not filling, use a full pixel's width
+    // if not filling, use thicker line
     //
-    cairo_set_line_width(cr, 1.0);
+    cairo_set_line_width(cr, LINE_THICK);
   }
   cairo_stroke(cr);
 
@@ -611,6 +617,7 @@ void rx_panadapter_update(RECEIVER *rx) {
   }
 #endif
 */
+
   if(display_sequence_errors) {
     if(sequence_errors!=0) {
       cairo_move_to(cr,100.0,50.0);
index 8804d01eaae8c886bb3ad7732331dffaf1782b88..10380fb0212c2784f8f169914a9b1d49696700fe 100644 (file)
@@ -51,7 +51,8 @@ static gboolean delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_d
 }
 
 static void step_select_cb (GtkToggleButton *widget, gpointer        data) {
-  if(gtk_toggle_button_get_active(widget)) {
+  int val=GPOINTER_TO_INT(data);
+  if(gtk_toggle_button_get_active(widget) && val >= 0 && val<STEPS) {
     step=steps[GPOINTER_TO_INT(data)];
     g_idle_add(ext_vfo_update,NULL);
   }
index b00746cf9584bd3678b77942bd65e6a3088fcf6a..49c4c394ece8e4fdfeb2340fd30c55161054567f 100644 (file)
@@ -85,7 +85,6 @@ static gboolean freqent_select_cb (GtkWidget *widget, gpointer data) {
   double  mult;
   long long f;
   static int set = 0;
-  SET_FREQUENCY *fp;
 
   // Instead of messing with LOCALE settings,
   // we print a "0.0" and look what the decimal
@@ -212,8 +211,6 @@ static void lock_cb(GtkWidget *widget, gpointer data) {
   g_idle_add(ext_vfo_update,NULL);
 }
 
-static GtkWidget *last_mode;
-
 void vfo_menu(GtkWidget *parent,int vfo) {
   int i;
 
@@ -258,7 +255,6 @@ void vfo_menu(GtkWidget *parent,int vfo) {
   gtk_misc_set_alignment (GTK_MISC (label), 1, .5);
   gtk_grid_attach(GTK_GRID(grid),label,0,1,3,1);
 
-  GtkWidget *step_rb=NULL;
   for (i=0; i<16; i++) {
     btn[i]=gtk_button_new_with_label(btn_labels[i]);
     set_button_text_color(btn[i],"black");
diff --git a/ext.c b/ext.c
index 789d7444c76e1d476f431c692b5b7c36e62da452..f0b798b3701cc8c0da65b5519d6b0c339409f7a0 100644 (file)
--- a/ext.c
+++ b/ext.c
@@ -194,7 +194,12 @@ void update_vfo_step(int direction) {
   for (i=0; i<STEPS; i++) {
     if (steps[i] == step) break;
   }
-  if (i >= STEPS) i=0;
+  //
+  // This should not happen (step size not found). If it happens,
+  // take the seond-smallest one such that the step size remains
+  // small after changing it.
+  //
+  if (i >= STEPS) i=1;
 
   if (direction > 0) {
     i++;
@@ -203,6 +208,7 @@ void update_vfo_step(int direction) {
   }
   if (i >= STEPS) i=STEPS;
   if (i < 0     ) i=0;
+  step=steps[i];
 
   vfo_update();
 }
@@ -787,7 +793,7 @@ int ext_remote_command(void *data) {
       temp=active_receiver->pan;
       int vfo=freq_command->id;
       long long f=ntohll(freq_command->hz);
-      local_set_frequency(vfo,f);
+      set_frequency(vfo,f);
       vfo_update();
       send_vfo_data(client,VFO_A);
       send_vfo_data(client,VFO_B);
index 083bc22b4f89f66f98b73172a8d5027dc35a3cb0..e925efa174a1af9abe1990ff27f73e6d2accac1d 100644 (file)
@@ -1095,9 +1095,10 @@ void process_ep2(uint8_t *frame)
             // for a combined attenuator/preamplifier with the AD9866 chip.
             // The value is between 0 and 60 and formally correspondes to
             // to an RX gain of -12 to +48 dB. However, we set here that
// a value of +16 (that is, 28 on the 0-60 scale) corresponds to
              // "zero attenuation"
-            chk_data(37 -(frame[4] & 0x3F) , rx_att[0], "RX1 HL ATT/GAIN");
+            // This means that the nominal value of "RX gain calibration" is 14.
+            chk_data(26 -(frame[4] & 0x3F) , rx_att[0], "RX1 HL ATT/GAIN");
            } else {
              chk_data((frame[4] & 0x1F) >> 0, rx_att[0], "RX1 ATT");
              chk_data((frame[4] & 0x20) >> 5, rx1_attE, "RX1 ATT enable");
index 44a339344399ebfa0bd31a41f66225149ec3214d..13558a9b248fa20d260e0b14da87f0fcb105fbaa 100644 (file)
@@ -98,13 +98,6 @@ EXTERN struct sockaddr_in addr_old;
 //
 EXTERN double c1,c2;
 
-//
-// Forward declarations for the audio functions
-//
-void audio_get_cards(void);
-void audio_open_output();
-void audio_write(int16_t, int16_t);
-
 //
 // Forward declarations for new protocol stuff
 //
@@ -1095,9 +1095,10 @@ void process_ep2(uint8_t *frame)
             // The value is between 0 and 60 and formally correspondes to
             // to an RX gain of -12 to +48 dB. However, we set here that
-             // a value of +16 (that is, 28 on the 0-60 scale) corresponds to
+             // a value of +14 (that is, 26 on the 0-60 scale) corresponds to
              // "zero attenuation"
index 91fcd4341749967df328021f905058ab728fa916..62865ad85c7c64e7b7043f0b242948c7ab8d74ca 100644 (file)